var express = require('express'),
    app = express(),
    morgan = require('morgan'),
    conf = require('config'),
    log = null,
    bodyParser = require('body-parser'),
    cluster = require('cluster'),
    mongoose = require('mongoose');

exports = module.exports;

exports.setup = function(options, callback) {
    log = configure_logging(options.log);

    var isClusterMaster = (cluster.isMaster && (process.env.NODE_CLUSTERED == 1));

    var is_http_thread = true;
    if (isClusterMaster ||
        ( 'undefined' !== typeof process.env.NODE_ISNOT_HTTP_SERVER_THREAD &&
        process.env.NODE_ISNOT_HTTP_SERVER_THREAD != 'true')) {
        is_http_thread = false;
    }

    log.info('is http thread? ' + is_http_thread);

    if (isClusterMaster) {
        require('nodebootstrap-clustering').setup();
    }

    if (is_http_thread) {
        app.listen(process.env.PORT || conf.app.port);
    }

// If we are not running a cluster at all:
    if (!isClusterMaster && cluster.isMaster) {
        log.info('Express server instance listening on port ' + conf.app.port);
    }

    module.parent.exports.setAppDefaults(app);

    mongoose.set('debug', conf.mongoose && conf.mongoose.debug);
    mongoose.connect(conf.db || '', conf.dbOptions || {}, function(err) {
        if (err) {
            log.error('Error:', err.message);
            return log.error('**Could not connect to MongoDB**');
        }

        callback(app);
    });

};

module.parent.exports.setAppDefaults = function(initapp) {

    // Use app passed in or create a new express app
    var someapp = initapp || express();

    someapp.use(require('compression')());

    // This logs the http requests to the console
    if (process.env.NODE_ENV === 'development') {
        someapp.use(morgan('dev'));
    }

    // parse application/x-www-form-urlencoded
    someapp.use(bodyParser.urlencoded({extended: true}))
    // parse application/json
    someapp.use(bodyParser.json());

    someapp.use(require('connect-multiparty')());
    someapp.use(require('method-override')('X-HTTP-Method-Override'));

    // Catch-all error handler. Modify as you see fit, but don't overuse.
    // Throwing exceptions is not how we normally handle errors in Node.
    someapp.use(catchAllErrorHandler);

    process.on('uncaughtException', function (err) {
        // handle the error safely
        log.fatal(err, err.stack);
        setTimeout(function() { // Give a chance for response to be sent, before killing the process
            process.exit(1);
        }, 500);
    });

    if (typeof initapp === 'undefined') {
        return someapp;
    }
};

function catchAllErrorHandler (err, req, res, next) {
    // Emergency: means system is unusable
    log.fatal({
        method: req.method,
        url: req.url,
        params: req.params,
        query: req.query,
        body: req.body,
        error: err,
        stack: err.stack
    });

    res
        .status(500)
        .end();

    // We aren't in the business of hiding exceptions under the rug. It should
    // still crush the process. All we want is: to properly log the error before
    // that happens.
    //
    // Clustering code in the lib/clustering module will restart the crashed process.
    // Make sure to always run clustering in production!
    setTimeout(function() { // Give a chance for response to be sent, before killing the process
        process.exit(1);
    }, 500);
}

function configure_logging (passedLog) {

    if (passedLog) {
        return passedLog;
    }

    return require('bunyan').createLogger({
        name: 'SuperWebServices'
    });
}
